@@ -6,7 +6,7 @@
  * Descr.: FD variable values management - C part                          *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ * Copyright (C) 1999-2012 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: fd_values_c.c,v 1.19 2011/03/28 16:20:05 diaz Exp $ */
+/* $Id: fd_values_c.c,v 1.21 2012/03/15 09:39:42 diaz Exp $ */
 
 #include <stdlib.h>
 
@@ -133,9 +133,9 @@ Prolog_Prototype(EXTRA_CSTR_ALT, 0);
 
   /* defined in fd_values_fd.fd */
 
-Bool pl_fd_domain(WamWord list_word, WamWord l_word, WamWord u_word);
+Bool pl_fd_domain(WamWord x_word, WamWord l_word, WamWord u_word);
 
-Bool pl_fd_domain_r(WamWord list_word, WamWord r_word);
+Bool pl_fd_domain_r(WamWord x_word, WamWord r_word);
 
 
 
