@@ -6,7 +6,7 @@
  * Descr.: mathematical support                                            *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ * Copyright (C) 1999-2012 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: math_supp.c,v 1.17 2011/03/28 16:20:05 diaz Exp $ */
+/* $Id: math_supp.c,v 1.19 2012/02/16 18:50:42 diaz Exp $ */
 
 #include <stdlib.h>
 #include <stdarg.h>
@@ -61,10 +61,10 @@
  * Constants                       *
  *---------------------------------*/
 
-#define DELAY_CSTR_STACK_SIZE      128
-#define VARS_STACK_SIZE            1024
+#define DELAY_CSTR_STACK_SIZE      1000
+#define VARS_STACK_SIZE            100000
 
-#define MAX_MONOMS                 1024
+#define MAX_MONOMS                 2000
 
 #define MAX_COEF_FOR_SORT          100
 
@@ -584,7 +584,7 @@ Push_Delayed_Cstr(int cstr, WamWord a1, WamWord a2, WamWord a3)
   res_word = Make_Self_Ref(H);
   Global_Push(res_word);
 
-  if (delay_sp - delay_cstr_stack == DELAY_CSTR_STACK_SIZE)
+  if (delay_sp - delay_cstr_stack >= DELAY_CSTR_STACK_SIZE)
     Pl_Err_Resource(pl_resource_too_big_fd_constraint);
 
   delay_sp->cstr = cstr;
@@ -724,7 +724,7 @@ Normalize(WamWord e_word, int sign, Poly *p)
 
   if (tag_mask == TAG_REF_MASK)
     {
-      if (vars_sp - vars_tbl == VARS_STACK_SIZE)
+      if (vars_sp - vars_tbl >= VARS_STACK_SIZE)
 	Pl_Err_Resource(pl_resource_too_big_fd_constraint);
 
       *vars_sp++ = word;
