@@ -6,7 +6,7 @@
  * Descr.: arithmetic (inline) management - C part                         *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ * Copyright (C) 1999-2012 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: arith_inl_c.c,v 1.28 2011/04/08 16:16:16 diaz Exp $ */
+/* $Id: arith_inl_c.c,v 1.30 2012/04/30 16:11:48 diaz Exp $ */
 
 #include <stdlib.h>
 #include <string.h>
@@ -390,6 +390,26 @@ Pl_Arith_Eval_2(WamWord exp_word, WamWord x_word)
 }
 
 
+/*-------------------------------------------------------------------------*
+ * PL_SUCC_2                                                               *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+Bool
+Pl_Succ_2(WamWord x_word, WamWord y_word)
+{
+  WamWord word, tag_mask;
+  PlLong x;
+
+  DEREF(x_word, word, tag_mask);
+  if (tag_mask != TAG_REF_MASK)
+    return Pl_Un_Positive_Check(Pl_Rd_Positive_Check(word) + 1, y_word);
+
+  Pl_Check_For_Un_Positive(word);
+  x = Pl_Rd_Positive_Check(y_word) - 1;
+  return x >= 0 && Pl_Get_Integer(x, word);
+}
+
+
 
 
 	  /* Mathematic Operations */
@@ -884,7 +904,7 @@ Pl_Fct_Max(WamWord x, WamWord y)
 WamWord FC
 Pl_Fct_Integer_Pow(WamWord x, WamWord y)
 {
-  IxItoI(x, y, Pl_Fct_Fast_Integer_Pow);
+  IFxIFtoIF(x, y, pow, Pl_Fct_Fast_Integer_Pow);
 }
 
 
