@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: b_params.h,v 1.20 2012/05/31 08:14:17 diaz Exp $ */
+/* $Id: b_params.h,v 1.21 2012/11/13 14:00:55 diaz Exp $ */
 
 #define MAX_VAR_NAME_LENGTH        1024
 
@@ -43,6 +43,11 @@
 
 #define MAX_SYS_VARS               256
 
+/* to return an hash as a Tagged INT use a modulo with HASH_MOD_VALUE
+ * it is the 1+max_integer on 32 bits 
+ */
+#define HASH_MOD_VALUE             (1 << (32 - TAG_SIZE - 1))
+
 
 /* ISO DTC2 seems to allow layout chars between minus sign and the number */
 #if 0
