@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: call.pl,v 1.20 2012/01/10 17:46:01 diaz Exp $ */
+/* $Id: call.pl,v 1.21 2012/11/13 14:00:55 diaz Exp $ */
 
 :-	built_in.
 
@@ -117,6 +117,13 @@ call_det(Goal, Deterministic) :-
 	'$call_internal'(P, CallInfo), !,
 	'$call_internal_with_cut'(Q, CallInfo, VarCut).
 
+	% P *-> Q alone (i.e. not inside a ;) is logically the same as P, Q. 
+        % However a cut in the test part (P) should be local to P (as in P -> Q).
+'$call_internal_with_cut'((P *-> Q), CallInfo, VarCut) :-
+	!,
+	'$call_internal'(P, CallInfo),
+	'$call_internal_with_cut'(Q, CallInfo, VarCut).
+
 '$call_internal_with_cut'(fail, _CallInfo, _VarCut) :-
 	!,
 	fail.
@@ -149,6 +156,13 @@ call_det(Goal, Deterministic) :-
 	;   '$call_internal_with_cut'(R, CallInfo, VarCut)
 	).
 
+'$call_internal_or'((P *-> Q), R, CallInfo, VarCut) :-
+	!,
+	(   '$call_internal'(P, CallInfo) *->
+	    '$call_internal_with_cut'(Q, CallInfo, VarCut)
+	;   '$call_internal_with_cut'(R, CallInfo, VarCut)
+	).
+
 '$call_internal_or'(P, _, CallInfo, VarCut) :-
 	'$call_internal_with_cut'(P, CallInfo, VarCut).
 
