@@ -6,7 +6,7 @@
  * Descr.: file consulting - C part                                        *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ * Copyright (C) 1999-2012 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -35,8 +35,9 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: consult_c.c,v 1.21 2011/03/28 16:20:05 diaz Exp $ */
+/* $Id: consult_c.c,v 1.24 2012/01/10 17:46:01 diaz Exp $ */
 
+#include <errno.h>
 #include <sys/types.h>
 
 #include "engine_pl.h"
@@ -105,8 +106,13 @@ Pl_Consult_2(WamWord tmp_file_word, WamWord pl_file_word)
 
   Pl_Flush_All_Streams();
   pid = Pl_M_Spawn_Redirect(arg, 0, pf_in, &f_out, &f_out);
-  Os_Test_Error(pid == -1);
-  if (pid == -2)
+
+  /* If pl2wam is not found we get ENOENT under Windows. 
+   * Under Unix the information is only obtained at Pl_M_Get_Status(). */
+
+  if (pid == -1 && errno != ENOENT)
+    Os_Test_Error(pid); /* ENOENT is for Windows */
+  if (pid < 0)
     {
     error_pl2wam:
       Pl_Err_System(Pl_Create_Atom("error trying to execute pl2wam "
