@@ -6,7 +6,7 @@
  * Descr.: control management                                              *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ * Copyright (C) 1999-2012 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: control.pl,v 1.17 2011/03/28 16:20:05 diaz Exp $ */
+/* $Id: control.pl,v 1.19 2012/04/30 16:11:48 diaz Exp $ */
 
 :-	built_in.
 
@@ -77,10 +77,15 @@ halt(X) :-
 
 
 
-for(I, L, U) :-
+between(L, U, X) :-
+	set_bip_name(between, 3),
+	'$call_c_test'('Pl_Between_3'(L, U, X)).
+
+'$between_alt' :-
+	'$call_c'('Pl_Between_Alt_0').
+
+
+
+for(X, L, U) :-			% deprecated: use between/3 instead
 	set_bip_name(for, 3),
-	'$call_c_test'('Pl_For_3'(I, L, U)).
-
-
-'$for_alt' :-
-	'$call_c'('Pl_For_Alt_0').
+	'$call_c_test'('Pl_Between_3'(L, U, X)).
