@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: error_supp.c,v 1.29 2012/01/10 17:46:01 diaz Exp $ */
+/* $Id: error_supp.c,v 1.30 2012/11/13 14:00:55 diaz Exp $ */
 
 #include <errno.h>
 #include <string.h>
@@ -256,10 +256,14 @@ Error_Supp_Initializer(void)
 void
 Pl_Set_Bip_Name_2(WamWord func_word, WamWord arity_word)
 {
+  PlLong arity;			/* use PlLong to avoid truncation */
   Pl_Set_C_Bip_Name("set_bip_name", 2);
 
   cur_bip_func = Pl_Rd_Atom_Check(func_word);
-  cur_bip_arity = Pl_Rd_Integer_Check(arity_word);
+  arity = Pl_Rd_Integer_Check(arity_word);
+  if (arity > MAX_ARITY)
+    arity = -1;
+  cur_bip_arity = arity;
 
   Pl_Unset_C_Bip_Name();
 
