@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: file_c.c,v 1.17 2012/01/10 17:46:01 diaz Exp $ */
+/* $Id: file_c.c,v 1.19 2012/11/22 13:23:40 diaz Exp $ */
 
 #include <string.h>
 
@@ -76,17 +76,45 @@
  *                                                                         *
  *-------------------------------------------------------------------------*/
 Bool
-Pl_Absolute_File_Name_2(WamWord f1_word, WamWord f2_word)
+Pl_Absolute_File_Name_2(WamWord path1_word, WamWord path2_word)
 {
-  char *f1, *f2;
+  char *path1, *path2;
 
-  f1 = pl_atom_tbl[Pl_Rd_Atom_Check(f1_word)].name;
+  path1 = pl_atom_tbl[Pl_Rd_Atom_Check(path1_word)].name;
 
-  f2 = Pl_M_Absolute_Path_Name(f1);
-  if (f2 == NULL)
-    Pl_Err_Domain(pl_domain_os_path, f1_word);
+  path2 = Pl_M_Absolute_Path_Name(path1);
+  if (path2 == NULL)
+    Pl_Err_Domain(pl_domain_os_path, path1_word);
 
-  return Pl_Un_String_Check(f2, f2_word);
+  return Pl_Un_String_Check(path2, path2_word);
+}
+
+
+
+
+/*-------------------------------------------------------------------------*
+ * PL_IS_ABSOLUTE_FILE_NAME_1                                              *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+Bool
+Pl_Is_Absolute_File_Name_1(WamWord path_word)
+{
+  char *path = pl_atom_tbl[Pl_Rd_Atom_Check(path_word)].name;
+
+  return Pl_M_Is_Absolute_File_Name(path);
+}
+
+
+
+
+/*-------------------------------------------------------------------------*
+ * PL_IS_RELATIVE_FILE_NAME_1                                              *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+Bool
+Pl_Is_Relative_File_Name_1(WamWord path_word)
+{
+  return !Pl_Is_Absolute_File_Name_1(path_word);
 }
 
 
@@ -98,67 +126,30 @@ Pl_Absolute_File_Name_2(WamWord f1_word, WamWord f2_word)
  *-------------------------------------------------------------------------*/
 Bool
 Pl_Decompose_File_Name_4(WamWord path_word, WamWord dir_word,
-		      WamWord prefix_word, WamWord suffix_word)
+			 WamWord prefix_word, WamWord suffix_word)
 {
-  char *path;
-  char *p;
-  int atom;
+  char *path = pl_atom_tbl[Pl_Rd_Atom_Check(path_word)].name;
+  char *dir, *base, *suffix;
   char c;
 
-  strcpy(pl_glob_buff, pl_atom_tbl[Pl_Rd_Atom_Check(path_word)].name);
-  path = pl_glob_buff;
-
-
   Pl_Check_For_Un_Atom(dir_word);
   Pl_Check_For_Un_Atom(prefix_word);
   Pl_Check_For_Un_Atom(suffix_word);
 
-  p = strrchr(path, DIR_SEP_C);
-#ifdef _WIN32
-  {
-    char *q = strrchr(path, '/');
+  dir = Pl_M_Decompose_File_Name(path, FALSE, &base, &suffix);
 
-    if (p == NULL || p > q)
-      p = q;
-  }
-#endif
-
-  if (p != NULL)
-    {
-      p++;
-      c = *p;
-      *p = '\0';
-      atom = Pl_Create_Allocate_Atom(path);
-      *p = c;
-      path = p;
-    }
-  else
-    atom = pl_atom_void;
-
-  if (!Pl_Get_Atom(atom, dir_word))
+  if (!Pl_Un_String(dir, dir_word))
     return FALSE;
 
+  c = *suffix;
+  *suffix = '\0';
 
-  p = strrchr(path, '.');
-  if (p != NULL)
-    {
-      *p = '\0';
-      atom = Pl_Create_Allocate_Atom(path);
-      *p = '.';
-      path = p;
-    }
-  else
-    atom = Pl_Create_Allocate_Atom(path);
-
-  if (!Pl_Get_Atom(atom, prefix_word))
+  if (!Pl_Un_String(base, prefix_word))
     return FALSE;
 
-  if (p != NULL)
-    atom = Pl_Create_Allocate_Atom(path);
-  else
-    atom = pl_atom_void;
+  *suffix = c;
 
-  return Pl_Get_Atom(atom, suffix_word);
+  return Pl_Un_String(suffix, suffix_word);
 }
 
 
@@ -169,55 +160,47 @@ Pl_Decompose_File_Name_4(WamWord path_word, WamWord dir_word,
  *                                                                         *
  *-------------------------------------------------------------------------*/
 Bool
-Pl_Prolog_File_Name_2(WamWord f1_word, WamWord f2_word)
+Pl_Prolog_File_Name_2(WamWord path1_word, WamWord path2_word)
 {
   int atom;
-  char *f1;
+  char *path1;
   int len;
   char *p;
   Bool suffix_pl;
 
-  atom = Pl_Rd_Atom_Check(f1_word);
-  f1 = pl_atom_tbl[atom].name;
+  atom = Pl_Rd_Atom_Check(path1_word);
+  path1 = pl_atom_tbl[atom].name;
 
-  f1 = Pl_M_Absolute_Path_Name(f1);
-  if (f1 == NULL)
-    Pl_Err_Domain(pl_domain_os_path, f1_word);
+  path1 = Pl_M_Absolute_Path_Name(path1);
+  if (path1 == NULL)
+    Pl_Err_Domain(pl_domain_os_path, path1_word);
 
-  if (strcmp(f1, "user") == 0)
+  if (strcmp(path1, "user") == 0)
     {
     same:
-      return Pl_Un_Atom_Check(atom, f2_word);
+      return Pl_Un_Atom_Check(atom, path2_word);
     }
 
-  p = strrchr(f1, DIR_SEP_C);
-#ifdef _WIN32
-  {
-    char *q = strrchr(f1, '/');
+  Find_Last_Dir_Sep(p, path1);
 
-    if (p == NULL || p > q)
-      p = q;
-  }
-#endif
-
-  if (strchr((p) ? p : f1, '.'))
+  if (strchr((p) ? p : path1, '.'))
     goto same;
 
-  strcpy(pl_glob_buff, f1);
-  len = strlen(f1);
+  strcpy(pl_glob_buff, path1);
+  len = strlen(path1);
 
   strcpy(pl_glob_buff + len, ".pl");
   suffix_pl = TRUE;
-  if (access(pl_glob_buff, F_OK))	/* f1.pl does not exist */
+  if (access(pl_glob_buff, F_OK))	/* path1.pl does not exist */
     {
       strcpy(pl_glob_buff + len, ".pro");
       suffix_pl = FALSE;
-      if (access(pl_glob_buff, F_OK))	/* f1.pro does not exist */
+      if (access(pl_glob_buff, F_OK))	/* path1.pro does not exist */
 	suffix_pl = TRUE;
     }
 
   sprintf(pl_glob_buff, "%s%s", pl_atom_tbl[atom].name,
 	  (suffix_pl) ? ".pl" : ".pro");
 
-  return Pl_Un_String_Check(pl_glob_buff, f2_word);
+  return Pl_Un_String_Check(pl_glob_buff, path2_word);
 }
