@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: flag_c.c,v 1.34 2012/05/31 08:14:17 diaz Exp $ */
+/* $Id: flag_c.c,v 1.35 2012/11/14 16:21:47 diaz Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -515,7 +515,7 @@ Unif_Flag(int i, WamWord value_word)
       break;
 
     case FLAG_MAX_ATOM:
-      n = MAX_ATOM;
+      n = pl_max_atom;
       break;
 
     case FLAG_MAX_UNGET:
