@@ -6,7 +6,7 @@
  * Descr.: list library                                                    *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ * Copyright (C) 1999-2012 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: list.pl,v 1.19 2011/03/28 16:20:05 diaz Exp $ */
+/* $Id: list.pl,v 1.22 2012/04/30 16:11:48 diaz Exp $ */
 
 :-	built_in.
 
@@ -60,7 +60,7 @@ append(L1, L2, L3) :-
 
 /*
 member(X, [H|T]) :-
-	(   X = H,
+	(   X = H
 	;   member(X, T)
 	).
 */
@@ -340,3 +340,100 @@ flatten(List, FlatList) :-
 
 '$flatten'(NonList, Tl, [NonList|Tl]).
 
+
+
+
+
+maplist(Goal, List) :-
+        '$maplist'(List, Goal).
+
+'$maplist'([], _).
+
+'$maplist'([X|List], Goal) :-
+        call(Goal, X),
+        '$maplist'(List, Goal).
+
+
+
+
+maplist(Goal, L1, L2) :-
+        '$maplist'(L1, L2, Goal).
+
+'$maplist'([], [], _).
+
+'$maplist'([X1|L1], [X2|L2], Goal) :-
+        call(Goal, X1, X2),
+        '$maplist'(L1, L2, Goal).
+
+
+
+
+maplist(Goal, L1, L2, L3) :-
+        '$maplist'(L1, L2, L3, Goal).
+
+'$maplist'([], [], [], _).
+
+'$maplist'([X1|L1], [X2|L2], [X3|L3], Goal) :-
+        call(Goal, X1, X2, X3),
+        '$maplist'(L1, L2, L3, Goal).
+
+
+
+
+maplist(Goal, L1, L2, L3, L4) :-
+        '$maplist'(L1, L2, L3, L4, Goal).
+
+'$maplist'([], [], [], [], _).
+
+'$maplist'([X1|L1], [X2|L2], [X3|L3], [X4|L4], Goal) :-
+        call(Goal, X1, X2, X3, X4),
+        '$maplist'(L1, L2, L3, L4, Goal).
+
+
+
+
+maplist(Goal, L1, L2, L3, L4, L5) :-
+        '$maplist'(L1, L2, L3, L4, L5, Goal).
+
+'$maplist'([], [], [], [], [], _).
+
+'$maplist'([X1|L1], [X2|L2], [X3|L3], [X4|L4], [X5|L5], Goal) :-
+        call(Goal, X1, X2, X3, X4, X5),
+        '$maplist'(L1, L2, L3, L4, L5, Goal).
+
+
+
+
+maplist(Goal, L1, L2, L3, L4, L5, L6) :-
+        '$maplist'(L1, L2, L3, L4, L5, L6, Goal).
+
+'$maplist'([], [], [], [], [], [], _).
+
+'$maplist'([X1|L1], [X2|L2], [X3|L3], [X4|L4], [X5|L5], [X6|L6], Goal) :-
+        call(Goal, X1, X2, X3, X4, X5, X6),
+        '$maplist'(L1, L2, L3, L4, L5, L6, Goal).
+
+
+
+
+maplist(Goal, L1, L2, L3, L4, L5, L6, L7) :-
+        '$maplist'(L1, L2, L3, L4, L5, L6, L7, Goal).
+
+'$maplist'([], [], [], [], [], [], [], _).
+
+'$maplist'([X1|L1], [X2|L2], [X3|L3], [X4|L4], [X5|L5], [X6|L6], [X7|L7], Goal) :-
+        call(Goal, X1, X2, X3, X4, X5, X6, X7),
+        '$maplist'(L1, L2, L3, L4, L5, L6, L7, Goal).
+
+
+
+
+maplist(Goal, L1, L2, L3, L4, L5, L6, L7, L8) :-
+        '$maplist'(L1, L2, L3, L4, L5, L6, L7, L8, Goal).
+
+'$maplist'([], [], [], [], [], [], [], [], _).
+
+'$maplist'([X1|L1], [X2|L2], [X3|L3], [X4|L4], [X5|L5], [X6|L6], [X7|L7], [X8|L8], Goal) :-
+        call(Goal, X1, X2, X3, X4, X5, X6, X7, X8),
+        '$maplist'(L1, L2, L3, L4, L5, L6, L7, L8, Goal).
+
