@@ -6,7 +6,7 @@
  * Descr.: scanner support                                                 *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ * Copyright (C) 1999-2012 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -35,13 +35,12 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: scan_supp.c,v 1.29 2011/06/10 13:58:09 diaz Exp $ */
+/* $Id: scan_supp.c,v 1.32 2012/06/20 13:07:35 diaz Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
-#include <setjmp.h>
 #include <stdarg.h>
 
 #define SCAN_SUPP_FILE
@@ -658,6 +657,15 @@ Scan_Quoted_Char(StmInf *pstm, Bool convert, int c0, Bool no_escape)
   if ((p = (char *) strchr(pl_escape_symbol, c)))	/* \a \b \f \n \r \t \v */
     return pl_escape_char[p - pl_escape_symbol];
 
+  if (!Flag_Value(FLAG_STRICT_ISO))
+    {
+      if (c == 's')		/* \s = space */
+	return ' ';
+
+      if (c == 'e')		/* ESCAPE */
+	return 27;
+    }
+
   if (c == 'x' || ('0' <= c && c <= '7'))	/* \xnn\ \nn\ */
     {
       if (c == 'x')
