@@ -6,7 +6,7 @@
  * Descr.: test - C part                                                   *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2009 Daniel Diaz                                     *
+ * Copyright (C) 1999-2012 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -25,8 +25,9 @@
 /* $Id: t_c.c,v 1.14 2002/05/07 17:45:47 diaz Exp $ */
 
 #include <stdio.h>
+#include <stdlib.h>
 
-#if 1
+#if 0
 #include "engine_pl.h"
 #include "bips_pl.h"
 #endif
@@ -171,3 +172,40 @@ Is_Kbd_Empty(void)
 #endif
 
 
+
+
+#if 0
+
+#include "gprolog.h"
+
+int sdl_init(PlTerm list)
+{
+  unsigned long flags=0;
+  int atom;
+  if(Pl_Builtin_List(list)) {
+    PlTerm *terms=NULL;
+    int len=Pl_List_Length(list);
+    terms=malloc(sizeof(PlTerm)*len);
+    if(terms) {
+      unsigned i;
+      Pl_Rd_Proper_List_Check(list, terms);
+
+      for(i=0; i<len; i++) {
+	char* s;
+	switch(Pl_Type_Of_Term(terms[i])) {
+	case PL_INT:
+	  printf("integer.\n");
+	  flags|=Pl_Rd_Integer(terms[i]);
+	  break;
+	case PL_ATM:
+	  atom = Pl_Rd_Atom(terms[i]);
+	  s=Pl_Atom_Name(atom);
+	  printf("atom %s is '%s'\n",s,Pl_Is_Valid_Atom(Pl_Rd_Atom(terms[i])) ? "valid":"invalid");
+	  break;
+	}
+      }
+    }
+  }
+  return PL_TRUE;
+}
+#endif
