@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: term_inl.pl,v 1.20 2012/01/10 17:46:01 diaz Exp $ */
+/* $Id: term_inl.pl,v 1.21 2012/11/13 14:00:55 diaz Exp $ */
 
 :-	built_in.
 
@@ -130,3 +130,13 @@ subsumes_term(General, Specific) :-
 acyclic_term(X) :-
 	set_bip_name(acyclic_term, 1),
 	'$call_c_test'('Pl_Acyclic_Term_1'(X)).
+
+
+
+term_hash(X, Depth, Range, Hash) :-
+	set_bip_name(term_hash, 4),
+	'$call_c_test'('Pl_Term_Hash_4'(X, Depth, Range, Hash)).
+
+term_hash(X, Hash) :-
+	set_bip_name(term_hash, 2),
+	'$call_c_test'('Pl_Term_Hash_2'(X, Hash)).
