@@ -6,7 +6,7 @@
  * Descr.: exception management (throw) - C part                           *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ * Copyright (C) 1999-2012 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: throw_c.c,v 1.17 2011/03/28 16:20:05 diaz Exp $ */
+/* $Id: throw_c.c,v 1.19 2012/01/10 17:46:01 diaz Exp $ */
 
 #include "engine_pl.h"
 #include "bips_pl.h"
@@ -95,8 +95,8 @@ Pl_Throw_2(WamWord ball_word, WamWord b_word)
 
       Pl_Stream_Printf(pstm, "\nsystem_error(cannot_catch_throw(");
       Pl_Write_Term(pstm, -1, MAX_PREC,
-		 WRITE_NUMBER_VARS | WRITE_NAME_VARS | WRITE_QUOTED,
-		 ball_word);
+		    WRITE_NUMBER_VARS | WRITE_NAME_VARS | WRITE_QUOTED, NULL,
+		    ball_word);
       Pl_Stream_Printf(pstm, "))\n");
       return;
     }
