@@ -6,7 +6,7 @@
  * Descr.: term writing support - header file                              *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ * Copyright (C) 1999-2012 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: write_supp.h,v 1.17 2011/03/28 16:20:05 diaz Exp $ */
+/* $Id: write_supp.h,v 1.20 2012/01/10 17:46:01 diaz Exp $ */
 
 /*---------------------------------*
  * Constants                       *
@@ -76,11 +76,13 @@ extern int pl_last_writing;
  * Function Prototypes             *
  *---------------------------------*/
 
-void Pl_Write_Term(StmInf *pstm, int depth, int prec, int mask,
-		WamWord term_word);
+void Pl_Write_Term(StmInf *pstm, int depth, int prec, int mask, WamWord *above_H,
+		   WamWord term_word);
 
 void Pl_Write_Simple(WamWord term_word);
 
+void Pl_Write_A_Full_Stop(StmInf *pstm);
+
 void Pl_Write_A_Char(StmInf *pstm, int c);
 
 char *Pl_Float_To_String(double d);
