@@ -6,7 +6,7 @@
  * Descr.: FD Range Implementation                                         *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ * Copyright (C) 1999-2012 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: fd_range.c,v 1.22 2011/03/28 16:20:05 diaz Exp $ */
+/* $Id: fd_range.c,v 1.24 2012/02/20 18:14:10 diaz Exp $ */
 
 #include <stdio.h>
 #include <string.h>
@@ -181,7 +181,7 @@ Pl_Vector_Nb_Elem(Vector vec)
 
   do
     {
-      nb_elem += Pl_Count_Set_Bits(*vec++);
+      nb_elem += Pl_Count_Set_Bits(*vec);
       vec++;
     }
   while (vec < end);
