@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: arch_dep.h,v 1.25 2012/06/20 13:07:35 diaz Exp $ */
+/* $Id: arch_dep.h,v 1.27 2012/11/22 13:23:40 diaz Exp $ */
 
 #ifndef _ARCH_DEP_H
 #define _ARCH_DEP_H
@@ -100,13 +100,41 @@
 
 #define DIR_SEP_S                  "\\"
 #define DIR_SEP_C                  '\\'
+#define DIR_SEP_C_ALT              '/'
 
-#else /* !defined(_WIN32) || defined(__CYGWIN__) */
+#elif defined(__CYGWIN__)
 
 #define DIR_SEP_S                  "/"
 #define DIR_SEP_C                  '/'
+#define DIR_SEP_C_ALT              '\\'
+
+#else  /* Unix */
+
+#define DIR_SEP_S                  "/"
+#define DIR_SEP_C                  '/'
+#define DIR_SEP_C_ALT              '/'
+
+#endif
+
+#define Is_Dir_Sep(c)              ((c) == DIR_SEP_C || (c) == DIR_SEP_C_ALT)
+
+
+
+#define Find_Last_Dir_Sep(_p, _path)			\
+  do {							\
+    char *_ptr;						\
+							\
+    for((_p) = NULL, _ptr = (_path); *_ptr; _ptr++)	\
+      if (Is_Dir_Sep(*_ptr))				\
+	(_p) = _ptr;					\
+  } while(0)
+
+
+
+#define Has_Drive_Specif(str) \
+  (((*(str) >= 'a' && *(str) <= 'z') || (*(str) >= 'A' && *(str) <= 'Z')) && (str)[1] == ':')
+
 
-#endif /* !defined(_WIN32) || defined(__CYGWIN__) */
 
 #if defined(M_ix86_cygwin) || defined(M_ix86_sco)
 #define Set_Line_Buf(s)            setvbuf(s, NULL, _IOLBF, 0)
