@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: atom.c,v 1.22 2012/01/10 17:46:01 diaz Exp $ */
+/* $Id: atom.c,v 1.25 2012/11/15 12:40:15 diaz Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -62,14 +62,11 @@
  * Constants                       *
  *---------------------------------*/
 
-#define ERR_ATOM_TBL_FULL          "Atom table full"
 #define ERR_ATOM_NIL_INVALID       "atom: invalid ATOM_NIL (should be %d)"
 
+#define ERR_TABLE_FULL_ENV         "Atom table full (max atom: %d, environment variable used: %s)"
 
-
-
-#define RADIX                      67
-#define INV_RADIX_MOD_MAX_ATOM     281707	/* see prog. euclide.c */
+#define ERR_TABLE_FULL_NO_ENV      "Atom table full (max atom: %d - fixed size)"
 
 
 
@@ -82,6 +79,8 @@
  * Global Variables                *
  *---------------------------------*/
 
+    /* this variable can be overwritten by top_comp.c (similarl to stacks) */
+
 int pl_char_type[256] = {
 
 /* nul soh stx etx eot enq ack bel bs  ht  nl  vt  np  cr  so  si  */
@@ -117,11 +116,6 @@ char pl_escape_char[] = "\a\b\f\n\r\t\v";
 
 static char str_char[256][2];
 
-static int hash_weight_tbl[256];
-static int hash_inv_tbl[RADIX];
-
-static char gen_sym_buff[1024];
-
 
 
 
@@ -129,11 +123,15 @@ static char gen_sym_buff[1024];
  * Function Prototypes             *
  *---------------------------------*/
 
-static AtomInf *Locate_Atom(char *name);
+static int Add_Atom(char *name, int len, unsigned hash, 
+		    AtomInf *patom, Bool allocate);
 
-static int Hash_String(char *str);
 
-static char *Gen_Sym(char *prefix, int gen_sym_hash);
+static AtomInf *Locate_Atom(char *name, unsigned hash);
+
+static unsigned Hash_String(char *str, int len);
+
+static void Error_Table_Full(void);
 
 
 
@@ -146,48 +144,19 @@ void
 Pl_Init_Atom(void)
 {
   int i, c;
+  
+  if (pl_max_atom < 256)
+    pl_max_atom = 256;
 
-  for (i = 0; i < 256; i++)
-    hash_weight_tbl[i] = i % RADIX;
+  if (pl_max_atom <= ATOM_NIL)
+    pl_max_atom = ATOM_NIL + 1;	/* to be sure h([]) % pl_max_atom == ATOM_NIL */
 
-  for (i = 0; i < 10; i++)
-    {
-      hash_weight_tbl[i + '0'] = i;
-      hash_inv_tbl[i] = i + '0';
-    }
+  if (pl_max_atom > (1 << ATOM_MAX_BITS)) /* be sure f/n words can be encoded (see wam_inst.h) */
+    pl_max_atom = (1 << ATOM_MAX_BITS);
 
-  for (i = 0; i < 26; i++)
-    {
-      hash_weight_tbl[i + 'A'] = i + 10;
-      hash_inv_tbl[i + 10] = i + 'A';
-    }
-
-
-  for (i = 0; i < 26; i++)
-    {
-      hash_weight_tbl[i + 'a'] = i + 10 + 26;
-      hash_inv_tbl[i + 10 + 26] = i + 'a';
-    }
-
-  i = 10 + 26 + 26;
-  hash_weight_tbl['#'] = i;
-  hash_inv_tbl[i] = '#';
-
-  i++;
-  hash_weight_tbl['$'] = i;
-  hash_inv_tbl[i] = '$';
-
-  i++;
-  hash_weight_tbl['&'] = i;
-  hash_inv_tbl[i] = '&';
-
-  i++;
-  hash_weight_tbl['_'] = i;
-  hash_inv_tbl[i] = '_';
-
-  i++;
-  hash_weight_tbl['@'] = i;
-  hash_inv_tbl[i] = '@';
+  pl_atom_tbl = (AtomInf *) Calloc(pl_max_atom, sizeof(AtomInf));
+  pl_nb_atom = 0;
+    
 
   for (c = 128; c < 256; c++) 
     {
@@ -198,9 +167,6 @@ Pl_Init_Atom(void)
     pl_char_conv[i] = i;
 
 
-
-  pl_nb_atom = 0;
-
   for (i = 0; i < 256; i++)
     {
       str_char[i][0] = i;
@@ -234,20 +200,11 @@ Pl_Init_Atom(void)
 int
 Pl_Create_Allocate_Atom(char *name)
 {
-  AtomInf *patom;
-  char *name1;
+  int len = strlen(name);
+  unsigned hash = Hash_String(name, len);
+  AtomInf *patom = Locate_Atom(name, hash);
 
-  patom = Locate_Atom(name);
-
-  if (patom == NULL)
-    Pl_Fatal_Error(ERR_ATOM_TBL_FULL);
-
-  if (patom->name != NULL)
-    return patom - pl_atom_tbl;	/* already exists */
-
-  name1 = Strdup(name);
-
-  return Pl_Create_Atom(name1);
+  return Add_Atom(name, len, hash, patom, TRUE);
 }
 
 
@@ -261,25 +218,45 @@ Pl_Create_Allocate_Atom(char *name)
 int
 Pl_Create_Atom(char *name)
 {
-  AtomInf *patom;
+  int len = strlen(name);
+  unsigned hash = Hash_String(name, len);
+
+  AtomInf *patom = Locate_Atom(name, hash);
+
+  return Add_Atom(name, len, hash, patom, FALSE);
+}
+
+
+
+
+/*-------------------------------------------------------------------------*
+ * ADD_ATOM                                                                *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+static int
+Add_Atom(char *name, int len, unsigned hash, AtomInf *patom, Bool allocate)
+{
   AtomProp prop;
   char *p;
   int c_type;
-  int lg;
   Bool identifier;
   Bool graphic;
-
-  patom = Locate_Atom(name);
+  patom = Locate_Atom(name, hash);
 
   if (patom == NULL)
-    Pl_Fatal_Error(ERR_ATOM_TBL_FULL);
+    Error_Table_Full();
 
   if (patom->name != NULL)
     return patom - pl_atom_tbl;	/* already exists */
 
+  if (allocate)
+    name = Strdup(name);
+
   pl_nb_atom++;
 
   patom->name = name;
+  patom->hash = hash;
+
   prop.needs_scan = FALSE;
 
   identifier = graphic = (*name != '\0');
@@ -298,11 +275,11 @@ Pl_Create_Atom(char *name)
 	prop.needs_scan = TRUE;
     }
 
-  prop.length = lg = p - name;
+  prop.length = len;
 
 #ifndef NO_USE_LINEDIT
-  if (lg > 1 && identifier)
-    Pl_LE_Compl_Add_Word(name, lg);
+  if (len > 1 && identifier)
+    Pl_LE_Compl_Add_Word(name, len);
 #endif
 
   if (pl_char_type[(unsigned char) *name] != SL)	/* small letter */
@@ -320,17 +297,17 @@ Pl_Create_Atom(char *name)
     {
       prop.type = GRAPHIC_ATOM;
       prop.needs_quote =
-	(lg == 1 && *name == '.') ||
-	(lg == 1 && *name == '%') ||
-	(lg >= 2 && name[0] == '/' && name[1] == '*')
+	(len == 1 && *name == '.') ||
+	(len == 1 && *name == '%') ||
+	(len >= 2 && name[0] == '/' && name[1] == '*')
 #if 0				/* this one does not need quotes it seems */
-	|| (lg == 2 && name[0] == '*' && name[1] == '/')
+	|| (len == 2 && name[0] == '*' && name[1] == '/')
 #endif
 	;
       goto finish;
     }
 
-  if (lg == 1 && pl_char_type[(unsigned char) *name] == SC)
+  if (len == 1 && pl_char_type[(unsigned char) *name] == SC)
     {
       prop.type = SOLO_ATOM;
       prop.needs_quote = (*name == ',');
@@ -339,8 +316,8 @@ Pl_Create_Atom(char *name)
 
   prop.type = OTHER_ATOM;
   prop.needs_quote = prop.needs_scan ||
-    !(lg == 2 && ((name[0] == '[' && name[1] == ']') ||
-		  (name[0] == '{' && name[1] == '}')));
+    !(len == 2 && ((name[0] == '[' && name[1] == ']') ||
+		   (name[0] == '{' && name[1] == '}')));
 
 
 finish:
@@ -354,7 +331,7 @@ finish:
 
 
 /*-------------------------------------------------------------------------*
- * PL_CREATE_ATOM                                                          *
+ * PL_CREATE_ATOM_TAGGED                                                   *
  *                                                                         *
  * Called by compiled prolog code.                                         *
  *-------------------------------------------------------------------------*/
@@ -375,9 +352,11 @@ Pl_Create_Atom_Tagged(char *name)
 int
 Pl_Find_Atom(char *name)
 {
+  int len = strlen(name);
+  unsigned hash = Hash_String(name, len);
   AtomInf *patom;
 
-  patom = Locate_Atom(name);
+  patom = Locate_Atom(name, hash);
   return (patom == NULL || patom->name == NULL) ? -1 : patom - pl_atom_tbl;
 }
 
@@ -388,35 +367,59 @@ Pl_Find_Atom(char *name)
  * LOCATE_ATOM                                                             *
  *                                                                         *
  * We use a specific hash table for atoms that cannot be extended but which*
- * provides a unique integer (in 0..MAX_ATOM-1) that could be used in the  *
+ * provides a unique integer (0..pl_max_atom-1) that could be used in the  *
  * future for tagged ATM words (and for structures).                       *
  *                                                                         *
+ * index (in the table) = hash code % pl_max_atom                          *
+ *                                                                         *
  * return the address of the found atom (if exists)                        *
  *        the address of the corresponding free cell (if not exist)        *
  *        NULL if the table is full                                        *
  *-------------------------------------------------------------------------*/
 static AtomInf *
-Locate_Atom(char *name)
+Locate_Atom(char *name, unsigned hash)
 {
-  int n;
-  AtomInf *patom, *endt;
+  int index;
+  AtomInf *patom0, *patom, *endt;
 
-  if (pl_nb_atom == MAX_ATOM)
-    return NULL;
+  index = hash % pl_max_atom;
+  patom = patom0 = pl_atom_tbl + index;
+  endt = pl_atom_tbl + pl_max_atom;
 
-  n = Hash_String(name);
-  /* here either the atom is in the table */
-  /* or there is at least one free cell.  */
-  patom = pl_atom_tbl + n;
-  endt = pl_atom_tbl + MAX_ATOM;
-
-  while (patom->name && strcmp(patom->name, name) != 0)
+  while (patom->name && (patom->hash != hash || strcmp(patom->name, name) != 0))
     {
       patom++;
       if (patom == endt)
 	patom = pl_atom_tbl;
+      if (patom == patom0)	/* one complete round: the table is full */
+	return NULL;
     }
 
+#if 0
+  if (patom->name == NULL)
+    {
+      if (patom != patom0)
+	{
+	  printf("atom: (%s) collision ixd: %ld -> %ld\n", name, patom0 - pl_atom_tbl, patom - pl_atom_tbl);
+	}
+
+      if (hash !=  patom - pl_atom_tbl)
+	{
+	  printf("atom: (%s)   hash: %u   idx: %ld\n", name, hash, patom - pl_atom_tbl);
+	}
+    }
+#endif
+
+#if 0
+  if (patom->name == NULL)
+    {
+      if (hash != index)
+	{
+	  printf("atom: (%s)   hash: %u   initial idx: %d\n", name, hash, index);
+	}
+    }
+#endif
+
   return patom;
 }
 
@@ -427,30 +430,27 @@ Locate_Atom(char *name)
 /*-------------------------------------------------------------------------*
  * HASH_STRING                                                             *
  *                                                                         *
- * This function computes a hash key from a string. If you modify MAX_ATOM *
- * or RADIX modify INV_RADIX_MOD_MAX_ATOM (see prog. euclide.c). MAX_ATOM  *
- * must be > 255. RADIX must be a prime number <256, the current value (67)*
- * does not need to be modified (or else update hash_weight_tbl and        *
- * hash_inv_tbl to control characters produced by Gen_Sym()).              *
+ * This function computes a hash key from a string.                        *
  *-------------------------------------------------------------------------*/
-static int
-Hash_String(char *str)
+static unsigned
+Hash_String(char *str, int len)
 {
-  int l = strlen(str);
-  char *p = str + l;
-  unsigned n = 0;
-
 #ifdef OPTIM_1_CHAR_ATOM
-  if (l == 1)			/* for 1 char strings: key = char */
-    return (int) ((unsigned char) (*str));
+  if (len == 1)			/* for 1 char strings: key = char */
+    return (unsigned) ((unsigned char) (*str));
 #endif
 
-  while (--p >= str)
-    n = n * RADIX + hash_weight_tbl[(unsigned char) *p];
+#if 1 /* uncomment to force a given ATOM_NIL (e.g. 256 ?) */
+  if (len == 2 && str[0] == '[' && str[1] == ']')
+    return ATOM_NIL;
+#endif
 
-  n %= MAX_ATOM;
+#if 0
+  if (len == 2 && str[0] == '[' && str[1] == ']')
+    printf("Hash([]) = %d\n", Pl_Hash_Buffer(str, len));
+#endif
 
-  return n;
+  return Pl_Hash_Buffer(str, len);
 }
 
 
@@ -460,80 +460,114 @@ Hash_String(char *str)
  * PL_GEN_NEW_ATOM                                                         *
  *                                                                         *
  * Find a new atom (gensym) beginning by a given prefix.                   *
- * hash<0 for any input or the index of the free atom to produce.          *
  *-------------------------------------------------------------------------*/
 int
-Pl_Gen_New_Atom(char *prefix, int hash)
+Pl_Gen_New_Atom(char *prefix)
 {
-  AtomInf *patom;
+#define GEN_SYM_CHARS "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"
 
-  if (pl_nb_atom == MAX_ATOM)
-    Pl_Fatal_Error(ERR_ATOM_TBL_FULL);
+  static char gen_sym_chars[] = GEN_SYM_CHARS;
+  static char gen_sym_buff[1024];
 
-  if (hash < 0)
-    {
-      patom = pl_atom_tbl;
-      while (patom->name)
-	patom++;
-      hash = patom - pl_atom_tbl;
-    }
+#define Gen_Sym_Rand()  (gen_sym_rand_next = gen_sym_rand_next * 1103515245 + 12345, (gen_sym_rand_next / 65536 % 32768))
 
-  return Pl_Create_Allocate_Atom(Gen_Sym(prefix, hash));
-}
+  static unsigned gen_sym_rand_next = 1; /* a simple RNG independent from the main one */
 
+#define TRY_MAX 1		/* 1 seems better than anything else on average ! */
 
+#ifdef DEBUG
+  static int nb = 0;
+  static unsigned long sum_len = 0;
+  static unsigned max_len = 0;
+  static unsigned long sum_try = 0;
+  static unsigned max_try = 0;
+  int try_count = 0;
+  static double time0 = 0;
+  double time, tsec = 0.0;
+#endif
 
-
-/*-------------------------------------------------------------------------*
- * GEN_SYM                                                                 *
- *                                                                         *
- * returns a string beginning by a prefix st. its hash code is gen_sym_hash*
- *-------------------------------------------------------------------------*/
-static char *
-Gen_Sym(char *prefix, int gen_sym_hash)
-{
-  unsigned pl = strlen(prefix);
-  unsigned hp = Hash_String(prefix);
-  unsigned x, i;
-  unsigned radix_p;
+  int try_no = 0;
+  int len;
+  unsigned hash;
   char *str;
+  int c;
+  AtomInf *patom;
+  int atom;
 
-  strcpy(gen_sym_buff, prefix);
-  str = gen_sym_buff + pl;
-
-  x = (gen_sym_hash - hp) % MAX_ATOM;
-
-  radix_p = 1;			/* compute 1/RADIX**pl = (1/RADIX)**pl */
-  for (i = 0; i < pl; i++)
-    radix_p *= INV_RADIX_MOD_MAX_ATOM;
-  radix_p %= MAX_ATOM;
-
-
-  x *= radix_p;			/* x = x/(RADIX**pl) */
-  x %= MAX_ATOM;
-  /* decompose x wrt radix RADIX */
-  do
-    {
-      *str++ = hash_inv_tbl[x % RADIX];
-      x /= RADIX;
-    }
-  while (x);
-
-  *str = '\0';
+  if (pl_nb_atom >= pl_max_atom)
+    Error_Table_Full();
 
 
 #ifdef DEBUG
-  if (Hash_String(gen_sym_buff) != gen_sym_hash)
-    {
-      DBGPRINTF("Gensym prefix: (%s) wanted hash: %d\n", prefix,
-		gen_sym_hash);
-      DBGPRINTF("   new string: (%s)    new hash: %d\n", gen_sym_buff,
-		Hash_String(gen_sym_buff));
-    }
-
+  nb++;
+  /* printf("GEN_SYM PREFIX : %s\n", prefix); */
 #endif
 
-  return gen_sym_buff;
+  strcpy(gen_sym_buff, prefix);
+  str = gen_sym_buff + strlen(prefix);
+
+  for(;;)
+    {
+      c = Gen_Sym_Rand() % (sizeof(gen_sym_chars) - 1); /* NB: -1 for '\0' */
+      *str = gen_sym_chars[c];
+      str[1] = '\0';
+
+      len =  str - gen_sym_buff + 1;
+
+      hash = Hash_String(gen_sym_buff, len);
+
+#if 1
+      patom = Locate_Atom(gen_sym_buff, hash);
+#else
+      patom = pl_atom_tbl + (hash % pl_max_atom);
+#endif
+
+#ifdef DEBUG
+      try_count++;
+      /*      printf("GEN_SYM TRY %3d: %s   len: %d\n", try_count, gen_sym_buff, len); */
+#endif
+
+      if (patom->name == NULL)
+	break;
+
+      if (++try_no == TRY_MAX)
+	{
+#if 0
+	  c = Gen_Sym_Rand() % (sizeof(gen_sym_chars) - 1); /* NB: -1 for '\0' */
+	  *str++ = gen_sym_chars[c];
+#else
+	  str++;
+#endif
+	  try_no = 0;
+	}
+    }
+
+
+  atom = Add_Atom(gen_sym_buff, len, hash, patom, TRUE);
+
+#ifdef DEBUG
+  sum_try += try_count;
+  if (try_count > max_try)
+    max_try = try_count;
+  c = len - strlen(prefix);
+  sum_len += c;
+  if (c > max_len)
+    max_len = c;
+  if (nb % 1000 == 0)
+    {
+      time = (double) Pl_M_User_Time(); /* time needed for the last 1000 gensym */
+      tsec = (time - time0) / 1000.0;
+      time0 = time;
+      printf("GENSYM #%5d: %s len:%d  len add:%d  (avg:%d  max:%d)  try:%d (avg:%d max:%d) time:%.3f\n", 
+	     nb, gen_sym_buff, (int) strlen(gen_sym_buff), c, 
+	     (int) (sum_len / nb), max_len,
+	     try_count, (int) (sum_try / nb), max_try,
+	     tsec);
+    }
+#endif
+
+
+  return atom;
 }
 
 
@@ -547,7 +581,7 @@ Gen_Sym(char *prefix, int gen_sym_hash)
 int
 Pl_Find_Next_Atom(int last_atom)
 {
-  while (++last_atom < MAX_ATOM)
+  while (++last_atom < pl_max_atom)
     {
       if (pl_atom_tbl[last_atom].name)
 	return last_atom;
@@ -555,3 +589,18 @@ Pl_Find_Next_Atom(int last_atom)
 
   return -1;
 }
+
+
+
+/*-------------------------------------------------------------------------*
+ * ERROR_TABLE_FULL                                                        *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+static void
+Error_Table_Full(void)
+{
+  if (pl_fixed_sizes)
+    Pl_Fatal_Error(ERR_TABLE_FULL_NO_ENV, pl_max_atom);
+  else
+    Pl_Fatal_Error(ERR_TABLE_FULL_ENV, pl_max_atom, ENV_VAR_MAX_ATOM);
+}
