@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: atom.h,v 1.19 2012/01/10 17:46:01 diaz Exp $ */
+/* $Id: atom.h,v 1.22 2012/11/15 12:40:15 diaz Exp $ */
 
 #if 1
 #define OPTIM_1_CHAR_ATOM
@@ -66,8 +66,10 @@
 
 
 
+/* NB: (hash) atom table size should not be < ATOM_NIL (else module will change it) */
+
+#define ATOM_NIL                 256
 
-#define ATOM_NIL                  1766
 
 
 
@@ -81,9 +83,9 @@
 
 
 
-#define Is_Valid_Code(c)           ((PlULong) (c)-1 <256-1)     /* 1<= c <256 */
-#define Is_Valid_Byte(c)           ((PlULong) (c) <256)	/* 0=< c <256 */
-#define Is_Valid_Atom(a)           ((PlULong) (a)<MAX_ATOM && \
+#define Is_Valid_Code(c)           ((PlULong) (c)-1 < 256-1)    /* 1 <= c < 256 */
+#define Is_Valid_Byte(c)           ((PlULong) (c) < 256)	/* 0 <= c < 256 */
+#define Is_Valid_Atom(a)           ((PlULong) (a) < pl_max_atom && \
                                     pl_atom_tbl[(a)].name!=NULL)
 
 
@@ -109,7 +111,9 @@ AtomProp;
 typedef struct			/* Atom information               */
 {				/* ------------------------------ */
   char *name;			/* key is <name> (the string)     */
+  unsigned hash;		/* the hash code of string (name) */
   AtomProp prop;		/* associated properties          */
+  void *info;			/* an user info (used by g_var)   */
 }
 AtomInf;
 
@@ -122,7 +126,8 @@ AtomInf;
 
 #ifdef ATOM_FILE
 
-AtomInf pl_atom_tbl[MAX_ATOM];
+AtomInf *pl_atom_tbl;
+int pl_max_atom;
 int pl_nb_atom;
 
 int pl_atom_void;
@@ -146,7 +151,8 @@ char pl_char_conv[256];
 
 #else
 
-extern AtomInf pl_atom_tbl[];
+extern AtomInf *pl_atom_tbl;
+extern int pl_max_atom;
 extern int pl_nb_atom;
 
 extern int pl_atom_void;
@@ -169,6 +175,7 @@ extern char pl_escape_char[];
 
 #endif
 
+PlLong pl_def_max_atom;		/* overwritten if needed (see top_comp.c) */
 
 
 
@@ -186,7 +193,7 @@ WamWord FC Pl_Create_Atom_Tagged(char *name);
 
 int Pl_Find_Atom(char *name);
 
-int Pl_Gen_New_Atom(char *prefix, int hash);
+int Pl_Gen_New_Atom(char *prefix);
 
 int Pl_Find_Next_Atom(int last_atom);
 
