@@ -2,13 +2,13 @@
 #define __GNU_PROLOG__        	1
 #define __GPROLOG__           	1
 #define __GPROLOG_MINOR__     	4
-#define __GPROLOG_PATCHLEVEL__	1
-#define __GPROLOG_VERSION__   	10401
+#define __GPROLOG_PATCHLEVEL__	2
+#define __GPROLOG_VERSION__   	10402
 
 #define PROLOG_DIALECT  	"gprolog"
 #define PROLOG_NAME     	"GNU Prolog"
-#define PROLOG_VERSION  	"1.4.1"
-#define PROLOG_DATE     	"Jun 28 2012"
+#define PROLOG_VERSION  	"1.4.2"
+#define PROLOG_DATE     	"Dec  3 2012"
 #define PROLOG_COPYRIGHT	"Copyright (C) 1999-2012 Daniel Diaz"
 
 #define PL_MIN_INTEGER		-268435456
