@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: machine.h,v 1.33 2012/06/20 13:07:35 diaz Exp $ */
+/* $Id: machine.h,v 1.35 2012/11/22 13:23:40 diaz Exp $ */
 
 #ifndef _MACHINE_H
 #define _MACHINE_H
@@ -91,6 +91,11 @@ Bool Pl_M_Set_Working_Dir(char *path);
 
 char *Pl_M_Absolute_Path_Name(char *src);
 
+Bool Pl_M_Is_Absolute_File_Name(char *path);
+
+char *Pl_M_Decompose_File_Name(char *path, Bool del_trail_slashes, 
+			       char **base, char **suffix);
+
 #if defined(_WIN32) && !defined(__CYGWIN__)
 
 int getpagesize(void);
@@ -191,7 +196,7 @@ void M_Check_Magic_Words(void); /* not compiled if not needed */
  * Malloc Management               *
  *---------------------------------*/
 
-#ifdef __OpenBSD__
+#if defined(__OpenBSD__) || defined(M_bsd)
 #define USE_DL_MALLOC
 #endif
 
