@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: pl_params.h,v 1.16 2012/01/10 17:46:02 diaz Exp $ */
+/* $Id: pl_params.h,v 1.17 2012/11/14 16:21:47 diaz Exp $ */
 
 #define MAX_OBJECT                 1024
 
@@ -43,8 +43,8 @@
 
 #define START_OPER_TBL_SIZE        1024
 
-#define ATOM_SIZE                  20
-#define MAX_ATOM                   (1 << ATOM_SIZE) /* number of elements */
+#define ENV_VAR_MAX_ATOM           "MAX_ATOM"
+#define DEFAULT_MAX_ATOM           32768
 
 #define NB_OF_X_REGS               256
 #define MAX_ARITY                  (NB_OF_X_REGS - 1)
