@@ -35,10 +35,14 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: stacks_sigsegv.c,v 1.4 2012/06/21 14:32:44 diaz Exp $ */
+/* $Id: stacks_sigsegv.c,v 1.6 2012/09/27 13:10:15 diaz Exp $ */
 
+/* copy this from try_sigaction.c */
 
+#if defined(M_ix86_sco)
+#define _XOPEN_SOURCE 700
 #define _XOPEN_SOURCE_EXTENDED
+#endif
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -59,7 +63,14 @@
 
 #include "engine_pl.h"
 
+ /* see configure.in */
+#if !defined(HAVE_WORKING_SIGACTION) && defined(LINUX_NEEDS_ASM_SIGCONTEXT)
+#include <asm/sigcontext.h>
+#endif
 
+#ifdef HAVE_SYS_SIGINFO_H
+#include <sys/siginfo.h>
+#endif
 
 #if defined(HAVE_MMAP) && !defined(_WIN32)
 #include <sys/mman.h>
@@ -69,10 +80,6 @@
 #endif
 #endif
 
- /* see configure.in */
-#if !defined(HAVE_WORKING_SIGACTION) && defined(LINUX_NEEDS_ASM_SIGCONTEXT)
-#include <asm/sigcontext.h>
-#endif
 
 
 #if 0
@@ -379,7 +386,7 @@ SIGSEGV_Handler(int sig, siginfo_t *sip)
 
 #elif defined(M_sparc_sunos)
 
-static static void
+static void
 SIGSEGV_Handler(int sig, int code, int scp, void *addr)
 {
   Handle_Bad_Address(addr);
@@ -387,7 +394,7 @@ SIGSEGV_Handler(int sig, int code, int scp, void *addr)
 
 #elif defined(M_alpha_osf)
 
-static static void
+static void
 SIGSEGV_Handler(int sig, int code, struct sigcontext *scp)
 {
   void *addr = (void *) (scp->sc_traparg_a0);
@@ -396,7 +403,7 @@ SIGSEGV_Handler(int sig, int code, struct sigcontext *scp)
 
 #elif defined(M_alpha_linux)
 
-static static void
+static void
 SIGSEGV_Handler(int sig, struct sigcontext scp)
 {
   void *addr = (void *) (scp.sc_fp_trigger_inst); /* why this one? */
@@ -406,7 +413,7 @@ SIGSEGV_Handler(int sig, struct sigcontext scp)
 
 #elif defined(M_ix86_linux)
 
-static static void
+static void
 SIGSEGV_Handler(int sig, struct sigcontext scp)
 {
   void *addr = (void *) scp.cr2;
@@ -415,7 +422,7 @@ SIGSEGV_Handler(int sig, struct sigcontext scp)
 
 #elif defined(M_powerpc_linux)
 
-static static void
+static void
 SIGSEGV_Handler(int sig, struct sigcontext scp)
 {
   void *addr = (void *) scp.regs->dar;
@@ -425,7 +432,7 @@ SIGSEGV_Handler(int sig, struct sigcontext scp)
 
 #elif defined(M_ix86_sco)
 #include <sys/siginfo.h>
-static static void
+static void
 SIGSEGV_Handler(int sig, siginfo_t * si)
 {
   void *addr = (void *) si->si_addr;
@@ -433,7 +440,7 @@ SIGSEGV_Handler(int sig, siginfo_t * si)
 }
 
 #elif defined(M_ix86_bsd) || defined(M_powerpc_bsd) || defined(M_sparc_bsd)
-static static void
+static void
 SIGSEGV_Handler(int sig, int code, struct sigcontext *scp)
 {
   void *addr = (void *) scp->sc_err;
