@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: wam_inst.h,v 1.27 2012/05/31 08:14:17 diaz Exp $ */
+/* $Id: wam_inst.h,v 1.28 2012/11/14 16:21:47 diaz Exp $ */
 
 #if 0
 #define GARBAGE_COLLECTOR
@@ -134,9 +134,11 @@
 
 	  /* Functor/arity */
 
-#define Functor_Arity(f, n)        (((n) << ATOM_SIZE) + (f))
-#define Functor_Of(word)           ((word) & (MAX_ATOM - 1))
-#define Arity_Of(word)             ((word) >> ATOM_SIZE)
+#define ATOM_MAX_BITS              20
+
+#define Functor_Arity(f, n)        (((n) << ATOM_MAX_BITS) | (f))
+#define Functor_Of(word)           ((word) & ((1 << ATOM_MAX_BITS) - 1))
+#define Arity_Of(word)             ((PlULong) (word) >> ATOM_MAX_BITS)
 
 
 #ifndef NO_USE_FD_SOLVER
