@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: test_noecho.c,v 1.18 2012/01/10 17:46:02 diaz Exp $ */
+/* $Id: test_noecho.c,v 1.19 2012/11/15 12:40:15 diaz Exp $ */
 
 #include <stdio.h>
 #include <ctype.h>
@@ -47,8 +47,10 @@
 
 #include "../EnginePl/set_locale.h"
 
-#include "../W32GUICons/w32gc_interf.h"
-#ifdef GUI_CONSOLE_WITH_STACK_SIZES       /* only to test GUI Console stack size dialog box */
+#include "../W32GUICons/w32gc_interf.h" /* only to test GUI Console memory size dialog box */
+int pl_max_atom;			/* to test the same dialog box */
+
+#ifdef GUI_CONSOLE_WITH_STACK_SIZES
 #define ENGINE_FILE                      /* to define stacks data */
 typedef PlLong WamWord;
 #include "../EnginePl/wam_stacks.h"
