@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: chkma.c,v 1.17 2012/06/20 13:07:35 diaz Exp $ */
+/* $Id: chkma.c,v 1.19 2012/11/15 12:40:15 diaz Exp $ */
 
 #include <stdio.h>
 #include <string.h>
@@ -44,6 +44,8 @@
 #define FC /* define FC to force arch_dep.h to no use FC */
 #endif
 
+int pl_max_atom;		/* to not need atom.o */
+
 #define IF_NO_FD_FILE
 //#include "engine_pl.h"
 #include "../EnginePl/engine.c"
@@ -1029,7 +1031,7 @@ void
 Pl_Allocate_Stacks(void)
 {
 }
-AtomInf pl_atom_tbl[1];
+AtomInf *pl_atom_tbl;
 void FC
 Pl_Create_Choice_Point(CodePtr codep_alt, int arity)
 {
