@@ -6,7 +6,7 @@
  * Descr.: instruction codification (needed for register allocation)       *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ * Copyright (C) 1999-2012 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: inst_codif.pl,v 1.18 2011/03/28 16:20:05 diaz Exp $ */
+/* $Id: inst_codif.pl,v 1.20 2012/05/31 08:14:18 diaz Exp $ */
 
           % alias stopping instructions
 
@@ -123,10 +123,12 @@ codif(execute(T), LCode) :-
 	( T = _ / N ; T = _:_/N ), !,
 	lst_r_for_call_execute(0, N, LCode).
 
-codif(load_cut_level(Tmp), [w(Tmp)]).
+codif(get_current_choice(x(Tmp)), [w(Tmp)]).
 
 codif(cut(x(Tmp)), [r(Tmp)]).
 
+codif(soft_cut(x(Tmp)), [r(Tmp)]).
+
 codif(call_c(_, LCOpt, LReg), LCode) :-
 	(   member(x(Tmp), LCOpt) ->
 	    End = [w(Tmp)]
