@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: read_file.pl,v 1.27 2012/05/31 08:14:18 diaz Exp $ */
+/* $Id: read_file.pl,v 1.29 2012/11/22 14:48:40 diaz Exp $ */
 
 /*-------------------------------------------------------------------------*
  * Data structures:                                                        *
@@ -167,13 +167,10 @@ read_file_error_nb(SynErrNb) :-
 
 
 open_new_prolog_file(PlFile0) :-
-	prolog_file_name(PlFile0, PlFile),
-	(   PlFile = user ->
-	    current_input(Stream)
-	;   open(PlFile, read, Stream)
-	),
 	g_read(open_file_stack, OpenFileStack),
-	g_assign(open_file_stack, [PlFile * Stream|OpenFileStack]),
+	prolog_file_name(PlFile0, PlFile),
+	open_new_prolog_file1(PlFile, OpenFileStack, PlFile1, Stream), !,
+	g_assign(open_file_stack, [PlFile1 * Stream|OpenFileStack]),
 	(   peek_char(Stream, '#'), % ignore #! starting line (for shebang support)
 	    repeat,
 	    get_char(Stream, X),
@@ -183,6 +180,39 @@ open_new_prolog_file(PlFile0) :-
 	).
 
 
+open_new_prolog_file1(user, _, user, Stream) :-
+	current_input(Stream).
+	
+open_new_prolog_file1(PlFile, _, PlFile, Stream) :-
+%	format('~n*** Trying to open ~a~n', [PlFile]),
+	catch(open(PlFile, read, Stream), error(existence_error(source_sink, _), _), fail).
+
+open_new_prolog_file1(PlFile, OpenFileStack, PlFile1, Stream) :-
+%	format('file stack: ~w~n', [OpenFileStack]),
+	is_relative_file_name(PlFile),
+	try_other_directory(OpenFileStack, PlFile, PlFile1, Stream).
+
+open_new_prolog_file1(PlFile, _, _, _) :-
+	throw(error(existence_error(source_sink, PlFile), open/3)).
+
+
+	/* If an included file is not found try to look in "parents" (includers) path.
+	 * If found return the new name (to have correct error msg and file_name in .wam)
+	 */
+
+try_other_directory([PlFile1 * _|_], PlFile, PlFile2, Stream) :-
+	decompose_file_name(PlFile1, Directory, _, _),
+	Directory \== '',
+	atom_concat(Directory, PlFile, PlFile2),
+%	format('   fail.~n+++ Trying to open ~a~n', [PlFile2]),
+	catch(open(PlFile2, read, Stream), _, fail).
+
+try_other_directory([_|OpenFileStack], PlFile, PlFile1, Stream) :-
+	try_other_directory(OpenFileStack, PlFile, PlFile1, Stream).
+
+
+
+	
 
 
 close_last_prolog_file :-
