@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: top_comp.c,v 1.36 2012/06/06 13:18:14 diaz Exp $ */
+/* $Id: top_comp.c,v 1.38 2012/11/20 09:15:46 diaz Exp $ */
 
 #include "../EnginePl/gp_config.h"
 
@@ -182,6 +182,7 @@ int pl_def_local_size = -1;
 int pl_def_global_size = -1;
 int pl_def_trail_size = -1;
 int pl_def_cstr_size = -1;
+int pl_def_max_atom = -1;
 int pl_fixed_sizes = 0;
 int needs_stack_file = 0;
 
@@ -399,6 +400,10 @@ Compile_Files(void)
         fprintf(fd, "long global pl_def_trail_size = %d\n", pl_def_trail_size);
       if (pl_def_cstr_size >= 0)
         fprintf(fd, "long global pl_def_cstr_size = %d\n", pl_def_cstr_size);
+
+      if (pl_def_max_atom >= 0)
+        fprintf(fd, "long global pl_def_max_atom = %d\n", pl_def_max_atom);
+
       if (pl_fixed_sizes)
         fprintf(fd, "long global pl_fixed_sizes = 1\n");
 
@@ -658,7 +663,6 @@ Link_Cmd(void)
       if (!Find_File(LIB_BIPS_FD, "", buff, 1) ||
           !Find_File(LIB_ENGINE_FD, "", buff + strlen(buff), 1))
         no_fd_lib = min_fd_bips = 1;
-      *buff = '\0';
     }
 
   if (file_name_out == NULL)
@@ -755,6 +759,8 @@ Link_Cmd(void)
     {    /* modify Linedit/Makefile.in to follow this list of ld objects */
       Find_File("w32gc_interf", OBJ_SUFFIX, buff + strlen(buff), 0);
       strcat(buff, " ");
+      Find_File("win_exe_icon", ".res", buff + strlen(buff), 1);
+      strcat(buff, " ");
 #ifdef _MSC_VER
       has_gui_console = 1;
 #endif
@@ -911,12 +917,12 @@ Find_File(char *file, char *suff, char *file_path, int ignore_error)
 {
   char name[MAXPATHLEN];
   char **pdev;
+  char *cur_end = file_path;
 
   sprintf(name, "%s%s", file, suff);
   if (!devel_mode)
     {
-      sprintf(file_path, "%s" DIR_SEP_S "lib" DIR_SEP_S "%s", start_path,
-	      name);
+      sprintf(file_path, "%s" DIR_SEP_S "lib" DIR_SEP_S "%s", start_path, name);
       if (access(file_path, F_OK) == 0)
 	return 1;
     }
@@ -932,6 +938,8 @@ Find_File(char *file, char *suff, char *file_path, int ignore_error)
 
   if (!ignore_error)
     Pl_Fatal_Error("cannot locate file %s", name);
+
+  *cur_end = '\0';
   return 0;
 }
 
@@ -1240,6 +1248,19 @@ Parse_Arguments(int argc, char *argv[])
 	      continue;
 	    }
 
+	  if (Check_Arg(i, "--max-atom"))
+	    {
+	      Record_Link_Warn_Option(i);
+	      if (++i >= argc)
+		Pl_Fatal_Error("SIZE missing after %s option", last_opt);
+	      pl_def_max_atom = strtol(argv[i], &q, 10);
+	      if (*q || pl_def_max_atom < 0)
+		Pl_Fatal_Error("invalid max atom (%s)", argv[i]);
+	      Record_Link_Warn_Option(i);
+	      needs_stack_file = 1;
+	      continue;
+	    }
+
 	  if (Check_Arg(i, "--fixed-sizes"))
 	    {
 	      Record_Link_Warn_Option(i);
@@ -1482,6 +1503,7 @@ Display_Help(void)
   L("  --global-size N             set default global stack size to N Kb");
   L("  --trail-size N              set default trail  stack size to N Kb");
   L("  --cstr-size N               set default cstr   stack size to N Kb");
+  L("  --max-atom N                set default atom   table size to N atoms");
   L("  --fixed-sizes               do not consult environment variables at run-time");
   L("  --gui-console               link the Win32 GUI console");
   L("  --no-top-level              do not link the top-level (force --no-debugger)");
