@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: w32_console.c,v 1.24 2012/01/10 17:46:02 diaz Exp $ */
+/* $Id: w32_console.c,v 1.26 2012/11/15 12:40:15 diaz Exp $ */
 
 #include <stdio.h>
 #include <ctype.h>
@@ -475,7 +475,8 @@ InitApplication(void)
   wc.lpszClassName = "w32_consoleWndClass";
   wc.lpszMenuName = MAKEINTRESOURCE(IDR_MENU);
   wc.hCursor = LoadCursor(NULL, IDC_ARROW);
-  wc.hIcon = LoadIcon(NULL, IDI_WINLOGO);
+  //  wc.hIcon = LoadIcon(NULL, IDI_WINLOGO);
+  wc.hIcon = LoadIcon(hInst, MAKEINTRESOURCE(IDI_ICON));
   if (!RegisterClass(&wc))
     return 0;
   return 1;
@@ -921,7 +922,9 @@ struct {
   { IDC_STACK_DESC0, IDC_DEF_SZ0, IDC_ENV_VAR_NAME0, IDC_ENV_SZ0, IDC_REG_SZ0, IDC_CUR_SZ0 },
   { IDC_STACK_DESC1, IDC_DEF_SZ1, IDC_ENV_VAR_NAME1, IDC_ENV_SZ1, IDC_REG_SZ1, IDC_CUR_SZ1 },
   { IDC_STACK_DESC2, IDC_DEF_SZ2, IDC_ENV_VAR_NAME2, IDC_ENV_SZ2, IDC_REG_SZ2, IDC_CUR_SZ2 },
-  { IDC_STACK_DESC3, IDC_DEF_SZ3, IDC_ENV_VAR_NAME3, IDC_ENV_SZ3, IDC_REG_SZ3, IDC_CUR_SZ3 }
+  { IDC_STACK_DESC3, IDC_DEF_SZ3, IDC_ENV_VAR_NAME3, IDC_ENV_SZ3, IDC_REG_SZ3, IDC_CUR_SZ3 },
+  /* for max_atom */
+  { IDC_STACK_DESC4, IDC_DEF_SZ4, IDC_ENV_VAR_NAME4, IDC_ENV_SZ4, IDC_REG_SZ4, IDC_CUR_SZ4 }
 };
 
 
@@ -958,7 +961,7 @@ StackSizesProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam)
       else
         {
           SetDlgItemText(hwndDlg, IDC_FIXED_SIZES,
-                         "New stack sizes will be taken into account at the next restart.");
+                         "New sizes will be taken into account at the next restart.");
         }
 
 
