@@ -6,7 +6,7 @@
  * Descr.: string dico management (file included by wam2ma.c and ma2asm.c) *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ * Copyright (C) 1999-2012 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: bt_string.c,v 1.16 2011/03/28 16:20:05 diaz Exp $ */
+/* $Id: bt_string.c,v 1.18 2012/04/30 16:11:48 diaz Exp $ */
 
 #include "bt_string.h"
 
@@ -55,7 +55,7 @@
  * Function Prototypes             *
  *---------------------------------*/
 
-static void BT_String_List_Rec(BTNode *bt_node, void (*fct) ());
+static void BT_String_List_Rec(BTNode *bt_node, BTStrLstFct fct);
 
 
 
@@ -142,7 +142,7 @@ BT_String_Lookup(BTString *bt_str, char *str)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 void
-BT_String_List(BTString *bt_str, void (*fct) ())
+BT_String_List(BTString *bt_str, BTStrLstFct fct)
 {
   BT_String_List_Rec(bt_str->tree, fct);
 }
@@ -155,12 +155,12 @@ BT_String_List(BTString *bt_str, void (*fct) ())
  *                                                                         *
  *-------------------------------------------------------------------------*/
 void
-BT_String_List_Rec(BTNode *bt_node, void (*fct) ())
+BT_String_List_Rec(BTNode *bt_node, BTStrLstFct fct)
 {
   if (bt_node == NULL)
     return;
 
   BT_String_List_Rec(bt_node->left, fct);
-  (*fct) (bt_node->no, bt_node->str, bt_node->info);
+  (*fct) (bt_node->no, bt_node->str, (void *) bt_node->info);
   BT_String_List_Rec(bt_node->right, fct);
 }
