@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: bt_string.h,v 1.17 2012/04/30 16:11:48 diaz Exp $ */
+/* $Id: bt_string.h,v 1.18 2012/09/26 13:47:02 diaz Exp $ */
 
 /*---------------------------------*
  * Constants                       *
@@ -51,7 +51,10 @@ typedef struct btnode
 {
   char *str;
   int no;
-  char info[32];		/* a buffer to store some information */
+#if WORD_SIZE == 64
+  int filler;		/* to preserve 64-bits align for info=PlLong (to avoid a SIGBUS) */
+#endif
+  char info[32];	/* a buffer to store some information */
   PBTNode left;
   PBTNode right;
 }
