@@ -6,7 +6,7 @@
  * Descr.: code generation - header file                                   *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2011 Daniel Diaz                                     *
+ * Copyright (C) 1999-2012 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -35,7 +35,7 @@
  * not, see http://www.gnu.org/licenses/.                                  *
  *-------------------------------------------------------------------------*/
 
-/* $Id: wam_protos.h,v 1.16 2011/03/28 16:20:05 diaz Exp $ */
+/* $Id: wam_protos.h,v 1.18 2012/05/31 08:14:18 diaz Exp $ */
 
 
 /*---------------------------------*
@@ -164,10 +164,14 @@ void F_retry(ArgVal arg[]);
 
 void F_trust(ArgVal arg[]);
 
-void F_load_cut_level(ArgVal arg[]);
+void F_pragma_arity(ArgVal arg[]);
+
+void F_get_current_choice(ArgVal arg[]);
 
 void F_cut(ArgVal arg[]);
 
+void F_soft_cut(ArgVal arg[]);
+
 void F_function(ArgVal arg[]);
 
 void F_call_c(ArgVal arg[]);
